'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _nodeEventstoreClient = require('node-eventstore-client');

var _nodeEventstoreClient2 = _interopRequireDefault(_nodeEventstoreClient);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('geteventstore:connectionManager');
var WAIT_DELAY = 50;
var connection = void 0;
var isConnecting = false;

var create = function () {
	var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(config) {
		var esConnection;
		return _regenerator2.default.wrap(function _callee$(_context) {
			while (1) {
				switch (_context.prev = _context.next) {
					case 0:
						(0, _assert2.default)(config.hostname, 'Hostname not provided');
						(0, _assert2.default)(config.port, 'Port not provided');

						if (!(connection && !isConnecting)) {
							_context.next = 6;
							break;
						}

						return _context.abrupt('return', connection);

					case 6:
						if (!isConnecting) {
							_context.next = 8;
							break;
						}

						return _context.abrupt('return', _bluebird2.default.delay(WAIT_DELAY).then(function () {
							return create(config);
						}));

					case 8:

						isConnecting = true;
						esConnection = _nodeEventstoreClient2.default.EventStoreConnection.create(config, 'tcp://' + config.hostname + ':' + config.port);


						esConnection.on('connected', function () {
							return debug('', 'Connection Connected');
						});
						esConnection.on('disconnected', function () {
							return debug('', 'Connection Disconnected');
						});
						esConnection.on('reconnecting', function () {
							return debug('', 'Connection Reconnecting...');
						});
						esConnection.on('closed', function (reason) {
							debug('', 'Connection Closed:', reason);
							if (esConnection === connection) {
								isConnecting = false;
								connection = undefined;
							}
						});
						esConnection.on('error', function (err) {
							debug('', 'Connection Error:', err.stack);
							console.error('EventStore: ' + err.stack);
							if (esConnection === connection) connection = undefined;
							try {
								isConnecting = false;
								esConnection.close();
							} catch (ex) {}
						});

						_context.prev = 15;
						_context.next = 18;
						return esConnection.connect();

					case 18:
						connection = esConnection;
						isConnecting = false;
						return _context.abrupt('return', esConnection);

					case 23:
						_context.prev = 23;
						_context.t0 = _context['catch'](15);

						connection = undefined;
						isConnecting = false;
						throw _context.t0;

					case 28:
					case 'end':
						return _context.stop();
				}
			}
		}, _callee, undefined, [[15, 23]]);
	}));

	return function create(_x) {
		return _ref.apply(this, arguments);
	};
}();

var closeAll = function closeAll() {
	return new _bluebird2.default(function (resolve, reject) {
		if (!connection) return resolve();

		var tempConnection = connection;
		connection.on('closed', function (reason) {
			debug('', 'Connection Closed:', reason);
			if (tempConnection && tempConnection === connection) {
				connection = undefined;
			}
			resolve();
		});
		connection.on('error', function (err) {
			if (tempConnection && tempConnection === connection) {
				connection = undefined;
			}
			try {
				tempConnection.close();
			} catch (ex) {}
			reject(err);
		});
		connection.on('connected', function () {
			debug('', 'Connection Connected:');
		});
		connection.close();
	});
};

var getConnections = function () {
	var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
		return _regenerator2.default.wrap(function _callee2$(_context2) {
			while (1) {
				switch (_context2.prev = _context2.next) {
					case 0:
						if (connection) {
							_context2.next = 2;
							break;
						}

						return _context2.abrupt('return', []);

					case 2:
						return _context2.abrupt('return', [connection]);

					case 3:
					case 'end':
						return _context2.stop();
				}
			}
		}, _callee2, undefined);
	}));

	return function getConnections() {
		return _ref2.apply(this, arguments);
	};
}();

exports.default = {
	create: create,
	closeAll: closeAll,
	getConnections: getConnections
};
module.exports = exports['default'];